#
# Copyright 2013 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from cyborgclient.common import utils
from cyborgclient.tests.unit import utils as test_utils


class CommonFiltersTest(test_utils.BaseTestCase):
    def test_limit(self):
        result = utils.common_filters(limit=42)
        self.assertEqual(['filters.field=limit', 'filters.value=42'], result)

    def test_limit_0(self):
        result = utils.common_filters(limit=0)
        self.assertEqual(['filters.field=limit', 'filters.value=0'], result)

    def test_limit_negative_number(self):
        result = utils.common_filters(limit=-2)
        self.assertEqual(['filters.field=limit', 'filters.value=-2'], result)

    def test_other(self):
        for key in ('marker', 'sort_key', 'sort_dir'):
            result = utils.common_filters(**{key: 'test'})
            self.assertEqual(['filters.field=%s' % key, 'filters.value=test'],
                             result)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import re
import sys
from unittest import mock

import fixtures
from keystoneauth1 import fixture
import six
from testtools import matchers

from cyborgclient import exceptions
import cyborgclient.shell
from cyborgclient.tests.unit import utils

FAKE_ENV = {'OS_USERNAME': 'username',
            'OS_PASSWORD': 'password',
            'OS_PROJECT_NAME': 'project_name',
            'OS_AUTH_URL': 'http://no.where/v2.0'}

FAKE_ENV2 = {'OS_USER_ID': 'user_id',
             'OS_PASSWORD': 'password',
             'OS_PROJECT_ID': 'project_id',
             'OS_AUTH_URL': 'http://no.where/v2.0'}

FAKE_ENV3 = {'OS_USERNAME': 'username',
             'OS_PASSWORD': 'password',
             'OS_PROJECT_ID': 'project_id',
             'OS_AUTH_URL': 'http://no.where/v2.0'}

FAKE_ENV4 = {'OS_USERNAME': 'username',
             'OS_PASSWORD': 'password',
             'OS_PROJECT_ID': 'project_id',
             'OS_USER_DOMAIN_NAME': 'Default',
             'OS_PROJECT_DOMAIN_NAME': 'Default',
             'OS_AUTH_URL': 'http://no.where/v3'}


def _create_ver_list(versions):
    return {'versions': {'values': versions}}


class ShellTest(utils.TestCase):
    AUTH_URL = utils.FAKE_ENV['OS_AUTH_URL']

    _msg_no_tenant_project = ("You must provide a project name or project id"
                              " via --os-project-name, --os-project-id,"
                              " env[OS_PROJECT_NAME] or env[OS_PROJECT_ID]")

    def setUp(self):
        super(ShellTest, self).setUp()
        self.nc_util = mock.patch(
            'cyborgclient.common.cliutils.isunauthenticated').start()
        self.nc_util.return_value = False

    def test_help_unknown_command(self):
        self.assertRaises(exceptions.CommandError, self.shell, 'help foofoo')

    def test_help(self):
        required = [
            '.*?^usage: ',
            '.*?^See "cyborg help COMMAND" for help on a specific command',
        ]
        stdout, stderr = self.shell('help')
        for r in required:
            self.assertThat((stdout + stderr),
                            matchers.MatchesRegex(r, re.DOTALL | re.MULTILINE))

    def test_help_no_options(self):
        required = [
            '.*?^usage: ',
            '.*?^See "cyborg help COMMAND" for help on a specific command.',
        ]
        stdout, stderr = self.shell('')
        for r in required:
            self.assertThat((stdout + stderr),
                            matchers.MatchesRegex(r, re.DOTALL | re.MULTILINE))

    def test_bash_completion(self):
        stdout, stderr = self.shell('bash-completion')
        # just check we have some output
        required = [
            '.*--fields',
            '.*help',
            '.*accelerator-list',
            '.*--help']
        for r in required:
            self.assertThat((stdout + stderr),
                            matchers.MatchesRegex(r, re.DOTALL | re.MULTILINE))

    def test_no_username(self):
        required = ('You must provide a username via either'
                    ' --os-username or via env[OS_USERNAME]')
        self.make_env(exclude='OS_USERNAME')
        try:
            self.shell('accelerator-list')
        except exceptions.CommandError as exc:
            self.assertEqual(required, exc.message)
        else:
            self.fail('CommandError not raised')

    def test_no_user_id(self):
        required = ('You must provide a username via'
                    ' either --os-username or via env[OS_USERNAME]')
        self.make_env(exclude='OS_USER_ID', fake_env=FAKE_ENV2)
        try:
            self.shell('accelerator-list')
        except exceptions.CommandError as exc:
            self.assertEqual(required, exc.message)
        else:
            self.fail('CommandError not raised')

    def test_no_project_name(self):
        required = self._msg_no_tenant_project
        self.make_env(exclude='OS_PROJECT_NAME')
        try:
            self.shell('accelerator-list')
        except exceptions.CommandError as exc:
            self.assertEqual(required, exc.message)
        else:
            self.fail('CommandError not raised')

    def test_no_project_id(self):
        required = self._msg_no_tenant_project
        self.make_env(exclude='OS_PROJECT_ID', fake_env=FAKE_ENV3)
        try:
            self.shell('accelerator-list')
        except exceptions.CommandError as exc:
            self.assertEqual(required, exc.message)
        else:
            self.fail('CommandError not raised')

    def test_no_auth_url(self):
        required = ('You must provide an auth url'
                    ' via either --os-auth-url or via env[OS_AUTH_URL]')
        self.make_env(exclude='OS_AUTH_URL')
        try:
            self.shell('accelerator-list')
        except exceptions.CommandError as exc:
            self.assertEqual(required, exc.message)
        else:
            self.fail('CommandError not raised')

    @mock.patch('cyborgclient.v1.client.Client')
    def test_service_type(self, mock_client):
        self.make_env()
        self.shell('accelerator-list')
        _, client_kwargs = mock_client.call_args_list[0]
        self.assertEqual('accelerator', client_kwargs['service_type'])

    @mock.patch('cyborgclient.v1.client.Client')
    def test_insecure(self, mock_client):
        self.make_env()
        self.shell('--insecure accelerator-list')
        _, session_kwargs = mock_client.call_args_list[0]
        self.assertEqual(True, session_kwargs['insecure'])

    @mock.patch('sys.stdin', side_effect=mock.MagicMock)
    @mock.patch('getpass.getpass', side_effect=EOFError)
    def test_no_password(self, mock_getpass, mock_stdin):
        required = ('You must provide a password via either --os-password, '
                    'env[OS_PASSWORD], or prompted response')
        self.make_env(exclude='OS_PASSWORD')
        try:
            self.shell('accelerator-list')
        except exceptions.CommandError as exc:
            self.assertEqual(required, exc.message)
        else:
            self.fail('CommandError not raised')

    @mock.patch('sys.argv', ['cyborg'])
    @mock.patch('sys.stdout', six.StringIO())
    @mock.patch('sys.stderr', six.StringIO())
    def test_main_noargs(self):
        # Ensure that main works with no command-line arguments
        try:
            cyborgclient.shell.main()
        except SystemExit:
            self.fail('Unexpected SystemExit')

        # We expect the normal usage as a result
        self.assertIn('Command-line interface to the OpenStack Cyborg API',
                      sys.stdout.getvalue())

    @mock.patch('cyborgclient.v1.client.Client')
    def _test_main_region(self, command, expected_region_name, mock_client):
        self.shell(command)
        mock_client.assert_called_once_with(
            api_version='latest', auth_token=None,
            auth_url=self.AUTH_URL, cloud=None, cyborg_url=None,
            insecure=False, interface='public', password='password',
            project_domain_id=None, project_domain_name=None, project_id=None,
            project_name='project_name', region_name=expected_region_name,
            service_type='accelerator', user_domain_id=None,
            user_domain_name=None, user_id=None, username='username')

    def test_main_option_region(self):
        self.make_env()
        self._test_main_region('--os-region-name=myregion accelerator-list',
                               'myregion')

    def test_main_env_region(self):
        fake_env = dict(utils.FAKE_ENV, OS_REGION_NAME='myregion')
        self.make_env(fake_env=fake_env)
        self._test_main_region('accelerator-list', 'myregion')

    def test_main_no_region(self):
        self.make_env()
        self._test_main_region('accelerator-list', None)

    @mock.patch('cyborgclient.v1.client.Client')
    def test_main_endpoint_public(self, mock_client):
        self.make_env()
        self.shell('--endpoint-type publicURL accelerator-list')
        mock_client.assert_called_once_with(
            username='username', password='password',
            interface='public', project_id=None,
            project_name='project_name', auth_url=self.AUTH_URL,
            service_type='accelerator', region_name=None,
            project_domain_id=None, project_domain_name=None,
            user_domain_id=None, user_domain_name=None,
            insecure=False, user_id=None, api_version='latest',
            auth_token=None, cyborg_url=None, cloud=None)

    @mock.patch('cyborgclient.v1.client.Client')
    def test_main_endpoint_internal(self, mock_client):
        self.make_env()
        self.shell('--endpoint-type internalURL accelerator-list')
        mock_client.assert_called_once_with(
            api_version='latest', auth_token=None,
            auth_url=self.AUTH_URL, cloud=None, cyborg_url=None,
            insecure=False, interface='internal', password='password',
            project_domain_id=None, project_domain_name=None, project_id=None,
            project_name='project_name', region_name=None,
            service_type='accelerator', user_domain_id=None,
            user_domain_name=None, user_id=None, username='username')


class ShellTestKeystoneV3(ShellTest):
    AUTH_URL = 'http://no.where/v3'

    def make_env(self, exclude=None, fake_env=FAKE_ENV):
        if 'OS_AUTH_URL' in fake_env:
            fake_env.update({'OS_AUTH_URL': self.AUTH_URL})
        env = dict((k, v) for k, v in fake_env.items() if k != exclude)
        self.useFixture(fixtures.MonkeyPatch('os.environ', env))

    def register_keystone_discovery_fixture(self, mreq):
        v3_url = "http://no.where/v3"
        v3_version = fixture.V3Discovery(v3_url)
        mreq.register_uri(
            'GET', v3_url, json=_create_ver_list([v3_version]),
            status_code=200)

    @mock.patch('cyborgclient.v1.client.Client')
    def test_main_endpoint_public(self, mock_client):
        self.make_env(fake_env=FAKE_ENV4)
        self.shell('--endpoint-type publicURL accelerator-list')
        mock_client.assert_called_once_with(
            username='username', password='password',
            interface='public', project_id='project_id',
            project_name=None, auth_url=self.AUTH_URL,
            service_type='accelerator', region_name=None,
            project_domain_id=None, project_domain_name='Default',
            user_domain_id=None, user_domain_name='Default',
            insecure=False, user_id=None, api_version='latest',
            auth_token=None, cyborg_url=None, cloud=None)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#

from cyborgclient.tests.unit.osc import fakes
from osc_lib.tests import utils
from unittest import mock
import uuid

deployable_created_at = '2019-06-24T00:00:00.000000+00:00'
deployable_updated_at = '2019-06-24T11:11:11.111111+11:11'
deployable_uuid = uuid.uuid4().hex
deployable_name = 'fake_dep_name'
deployable_parent_id = None
deployable_root_id = 1
deployable_num_accelerators = 4
deployable_device_id = 0

DEPLOYABLE = {
    'created_at': deployable_created_at,
    'updated_at': deployable_updated_at,
    'id': deployable_uuid,
    'name': deployable_name,
    'parent_id': deployable_parent_id,
    'root_id': deployable_root_id,
    'num_accelerators': deployable_num_accelerators,
    'device_id': deployable_device_id,
}

device_created_at = '2019-06-24T00:00:00.000000+00:00'
device_updated_at = '2019-06-24T22:22:22.222222+22:22'
device_id = 1
device_uuid = uuid.uuid4().hex
device_name = 'fake_dev_name'
device_type = 'fake_dev_type'
device_vendor = '0x8086'
device_model = 'fake_dev_model'
device_hostname = 'fake_host'
device_std_board_info = '{"product_id": "0x09c4"}'
device_vendor_board_info = 'fake_vb_info'

DEVICE = {
    'created_at': device_created_at,
    'updated_at': device_updated_at,
    'id': device_id,
    'uuid': device_uuid,
    'type': device_type,
    'vendor': device_vendor,
    'model': device_model,
    'hostname': device_hostname,
    'std_board_info': device_std_board_info,
    'vendor_board_info': device_vendor_board_info
}

device_profile_created_at = '2019-06-24T00:00:00.000000+00:00'
device_profile_updated_at = '2019-06-24T11:11:11.111111+11:11'
device_profile_id = 1
device_profile_uuid = uuid.uuid4().hex
device_profile_name = 'fake_devprof_name'
device_profile_groups = [
    {"resources:ACCELERATOR_FPGA": "1",
     "trait:CUSTOM_FPGA_INTEL_PAC_ARRIA10": "required",
     "trait:CUSTOM_FUNCTION_ID_3AFB": "required",
     },
    {"resources:CUSTOM_ACCELERATOR_FOO": "2",
     "resources:CUSTOM_MEMORY": "200",
     "trait:CUSTOM_TRAIT_ALWAYS": "required",
     }
]

DEVICE_PROFILE = {
    'created_at': device_profile_created_at,
    'updated_at': device_profile_updated_at,
    'id': device_profile_id,
    'uuid': device_profile_uuid,
    'name': device_profile_name,
    'groups': device_profile_groups,
}

accelerator_request_uuid = uuid.uuid4().hex
accelerator_request_state = 'fake_state'
accelerator_request_device_profile_name = 'fake_arq_devprof_name'
accelerator_request_hostname = 'fake_arq_hostname'
accelerator_request_device_rp_uuid = 1
accelerator_request_instance_uuid = 2
accelerator_request_attach_handle_type = 3
accelerator_request_attach_handle_info = 4

ACCELERATOR_REQUEST = {
    'uuid': accelerator_request_uuid,
    'state': accelerator_request_state,
    'device_profile_name': accelerator_request_device_profile_name,
    'hostname': accelerator_request_hostname,
    'device_rp_uuid': accelerator_request_device_rp_uuid,
    'instance_uuid': accelerator_request_instance_uuid,
    'attach_handle_type': accelerator_request_attach_handle_type,
    'attach_handle_info': accelerator_request_attach_handle_info,
}


class TestAccelerator(utils.TestCommand):

    def setUp(self):
        super(TestAccelerator, self).setUp()

        self.app.client_manager.auth_ref = mock.MagicMock(auth_token="TOKEN")
        self.app.client_manager.accelerator = mock.MagicMock()


class FakeAcceleratorResource(fakes.FakeResource):

    def get_keys(self):
        return {'property': 'value'}



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#


"""Cyborg v2 Acceleration accelerator action implementations"""
import logging

from openstack import exceptions as sdk_exc
from osc_lib.command import command
from osc_lib import utils as oscutils

from cyborgclient.common import utils
from cyborgclient import exceptions as exc
from cyborgclient.i18n import _


class ListDeployable(command.Lister):
    """List all deployables"""

    def get_parser(self, prog_name):
        parser = super(ListDeployable, self).get_parser(prog_name)
        parser.add_argument(
            '--long',
            dest='detail',
            action='store_true',
            default=False,
            help=_("List additional fields in output")
        )
        return parser

    def take_action(self, parsed_args):
        acc_client = self.app.client_manager.accelerator

        if parsed_args.detail:
            column_headers = (
                "created_at",
                "updated_at",
                "uuid",
                "parent_id",
                "root_id",
                "name",
                "num_accelerators",
                "device_id"
            )

            columns = (
                "created_at",
                "updated_at",
                "id",
                "parent_id",
                "root_id",
                "name",
                "num_accelerators",
                "device_id"
            )
        else:
            column_headers = (
                "uuid",
                "name",
                "device_id",
            )
            columns = (
                "id",
                "name",
                "device_id",
            )
        data = acc_client.deployables()
        if not data:
            return (), ()
        formatters = {}
        return (column_headers,
                (oscutils.get_item_properties(
                    s, columns, formatters=formatters) for s in data))


class ShowDeployable(command.ShowOne):
    """Show deployable details."""
    log = logging.getLogger(__name__ + ".ShowDeployable")

    def get_parser(self, prog_name):
        parser = super(ShowDeployable, self).get_parser(prog_name)
        parser.add_argument(
            "deployable",
            metavar="<uuid>",
            help=_("UUID of the deployable.")
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        acc_client = self.app.client_manager.accelerator
        return _show_deployable(acc_client,
                                parsed_args.deployable)


def _show_deployable(acc_client, uuid):
    """Show detailed info about deployable."""
    columns = (
        "created_at",
        "updated_at",
        "uuid",
        "name",
    )
    try:
        deployable = acc_client.get_deployable(uuid)
    except sdk_exc.ResourceNotFound:
        raise exc.CommandError(_('deployable not found: %s') % uuid)
    formatters = {
        'data': utils.json_formatter,
    }
    data = deployable.to_dict()
    data['uuid'] = data.pop('id', uuid)
    return columns, oscutils.get_dict_properties(data, columns,
                                                 formatters=formatters)


class ProgramDeployable(command.ShowOne):
    """Reconfigure deployable."""
    log = logging.getLogger(__name__ + ".ProgramDeployable")

    def get_parser(self, prog_name):
        parser = super(ProgramDeployable, self).get_parser(prog_name)
        parser.add_argument(
            'deployable_uuid',
            metavar='<deployable_uuid>',
            default=False,
            help=_("Deployable UUID for reconfigure.")
        )
        parser.add_argument(
            'image_uuid',
            metavar='<image_uuid>',
            default=False,
            help=_("Image UUID for reconfigure.")
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        acc_client = self.app.client_manager.accelerator
        dep_uuid = parsed_args.deployable_uuid
        try:
            acc_client.get_deployable(dep_uuid)
        except sdk_exc.ResourceNotFound:
            raise exc.CommandError(_('deployable not found: %s') % dep_uuid)

        image_uuid = parsed_args.image_uuid
        image_client = self.app.client_manager.image
        try:
            image_client.get(image_uuid)
        except sdk_exc.ResourceNotFound:
            raise exc.CommandError(_('image not found: %s') % image_uuid)

        program_info = [{'path': '/program',
                         'value': [{'image_uuid': image_uuid}],
                         'op': 'replace'}]
        acc_client.update_deployable(dep_uuid, program_info)
        return _show_deployable(acc_client, dep_uuid)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import connection as sdk_connection
import testtools
from unittest import mock

from cyborgclient.osc import plugin


class TestPlugin(testtools.TestCase):

    @mock.patch.object(sdk_connection, 'Connection')
    def test_create_connection_with_profile(self, mock_connection):
        class FakeService(object):
            interface = 'public'
            region = 'a_region'
            version = '2'
            api_version = None
            service_type = 'accelerator'

        mock_prof = mock.Mock()
        mock_prof.get_services.return_value = [FakeService()]
        mock_conn = mock.Mock()
        mock_connection.return_value = mock_conn
        kwargs = {
            'user_id': '123',
            'password': 'abc',
            'auth_url': 'test_url'
        }
        res = plugin.create_connection(mock_prof, **kwargs)
        mock_connection.assert_called_once_with(
            app_name=None, app_version=None,
            config=mock.ANY,
            accelerator_api_version=plugin.CURRENT_API_VERSION,
            **kwargs
        )
        self.assertEqual(mock_conn, res)

    @mock.patch.object(sdk_connection, 'Connection')
    def test_create_connection_without_profile(self, mock_connection):
        mock_conn = mock.Mock()
        mock_connection.return_value = mock_conn
        kwargs = {
            'interface': 'public',
            'region_name': 'RegionOne',
            'user_id': '123',
            'password': 'abc',
            'auth_url': 'test_url'
        }
        res = plugin.create_connection(**kwargs)

        mock_connection.assert_called_once_with(
            app_name=None, app_version=None,
            auth_url='test_url',
            accelerator_api_version=plugin.CURRENT_API_VERSION,
            config=None,
            interface='public',
            password='abc',
            region_name='RegionOne',
            user_id='123'
        )
        self.assertEqual(mock_conn, res)



#   Copyright (c) 2018 Intel, Inc. All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from cyborgclient.v1 import basemodels


CREATION_ATTRIBUTES = basemodels.CREATION_ATTRIBUTES


class Deployable(basemodels.BaseModel):
    model_name = "Deployable"


class DeployableManager(basemodels.BaseModelManager):
    api_name = "deployables"
    base_url = "accelerators/deployables"
    resource_class = Deployable

    def deallocation(self, deployable_uuid):
        """Delete specified accelerators from instance.

        :param deployable_uuid: deployable_uuid which is attached to the
        instance.
        """
        body = [{"op": "replace", "path": "/instance_uuid", "value": None}]
        resp = self.update(deployable_uuid, body)
        return resp

    def allocation(self, deployable_uuid, instance_uuid):
        body = [{"op": "replace", "path": "/instance_uuid",
                "value": instance_uuid}]
        resp = self.update(deployable_uuid, body)
        return resp

    def list(self, limit=None, marker=None, sort_key=None,
             sort_dir=None, **add_filters):
        """List accelerators.

        :param limit:The maximum number of results to return per
                      request, if:

            1) limit > 0, the maximum number of accelerators to return.
            2) limit == 0, return the entire list of accelerators.
            3) limit param is NOT specified (None), the number of items
               returned respect the maximum imposed by the Cyborg API
               (see Cyborg's api.max_limit option).
        :param sort_key: Optional, field used for sorting.

        :param sort_dir: Optional, direction of sorting, either 'asc' (the
                         default) or 'desc'.
        :param extra_filters: Optional, additional filter parameters for query
                         deployable, such as interface_type=pci, host=node-1.
        :return:A list of accelerators.
        """
        res = super(DeployableManager, self).list(limit=limit, marker=marker,
                                                  sort_key=sort_key,
                                                  sort_dir=sort_dir,
                                                  **add_filters)
        return res
